@@ -1,7 +1,7 @@
 /*
-*   $Id: args.c,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: args.c,v 1.4 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1999-2001, Darren Hiebert
+*   Copyright (c) 1999-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -20,7 +20,7 @@
 
 #include "args.h"
 #include "debug.h"
-#include "main.h"
+#include "routines.h"
 
 /*
 *   FUNCTION DEFINITIONS
@@ -71,6 +71,12 @@ static char* nextStringLine (const char** const next)
     }
     if (*end == '\n')
 	++end;
+    else if (*end == '\r')
+    {
+	++end;
+	if (*end == '\n')
+	    ++end;
+    }
     *next = end;
     return result;
 }
@@ -124,18 +130,27 @@ static char* nextFileLine (FILE* const fp)
 
 	Assert (fp != NULL);
 	c = fgetc (fp);
-	while (c != EOF  &&  c != '\n')
+	while (c != EOF)
 	{
-	    vStringPut (vs, c);
+	    if (c != '\n'  &&  c != '\r')
+		vStringPut (vs, c);
+	    else if (vStringLength (vs) > 0)
+		break;
 	    c = fgetc (fp);
 	}
-	vStringTerminate (vs);
-	if (vStringLength (vs) > 0)
+	if (c != EOF)
 	{
+	    if (c == '\r')
+	    {
+		c = fgetc (fp);
+		if (c != '\n')
+		    c = ungetc (c, fp);
+	    }
+	    vStringTerminate (vs);
 	    result = xMalloc (vStringLength (vs) + 1, char);
 	    strcpy (result, vStringValue (vs));
+	    vStringDelete (vs);
 	}
-	vStringDelete (vs);
     }
     return result;
 }
