@@ -1,7 +1,7 @@
 /*
-*   $Id: main.c,v 1.12 2002/06/17 04:48:13 darren Exp $
+*   $Id: main.c,v 1.20 2003/08/28 03:56:21 darren Exp $
 *
-*   Copyright (c) 1996-2002, Darren Hiebert
+*   Copyright (c) 1996-2003, Darren Hiebert
 *
 *   Author: Darren Hiebert <dhiebert@users.sourceforge.net>
 *           http://ctags.sourceforge.net
@@ -63,16 +63,16 @@
 # undef boolean
 #endif
 #ifdef HAVE_DIRECT_H
-# include <direct.h>	/* to _getcwd */
+# include <direct.h>	/* to _getcwd() */
 #endif
 #ifdef HAVE_DOS_H
 # include <dos.h>	/* to declare FA_DIREC */
 #endif
 #ifdef HAVE_DIR_H
-# include <dir.h>	/* to declare findfirst () and findnext () */
+# include <dir.h>	/* to declare findfirst() and findnext */
 #endif
 #ifdef HAVE_IO_H
-# include <io.h>	/* to declare _finddata_t in MSVC++ 4.x */
+# include <io.h>	/* to declare _findfirst() */
 #endif
 
 
@@ -91,7 +91,6 @@
 /*
 *   DATA DEFINITIONS
 */
-static stringList* Excluded = NULL;
 static struct { long files, lines, bytes; } Totals = { 0, 0, 0 };
 
 #ifdef AMIGA
@@ -114,61 +113,9 @@ static boolean createTagsForEntry (const char *const entryName);
 *   FUNCTION DEFINITIONS
 */
 
-extern vString *combinePathAndFile (
-    const char *const path, const char *const file)
-{
-    vString *const filePath = vStringNew ();
-#ifdef VMS
-    const char *const directoryId = strstr (file, ".DIR;1");
-
-    if (directoryId == NULL)
-    {
-	const char *const versionId = strchr (file, ';');
-
-	vStringCopyS (filePath, path);
-	if (versionId == NULL)
-	    vStringCatS (filePath, file);
-	else
-	    vStringNCatS (filePath, file, versionId - file);
-	vStringCopyToLower (filePath, filePath);
-    }
-    else
-    {
-	/*  File really is a directory; append it to the path.
-	 *  Gotcha: doesn't work with logical names.
-	 */
-	vStringNCopyS (filePath, path, strlen (path) - 1);
-	vStringPut (filePath, '.');
-	vStringNCatS (filePath, file, directoryId - file);
-	if (strchr (path, '[') != NULL)
-	    vStringPut (filePath, ']');
-	else
-	    vStringPut (filePath, '>');
-	vStringTerminate (filePath);
-    }
-#else
-    const int lastChar = path [strlen (path) - 1];
-# ifdef MSDOS_STYLE_PATH
-    boolean terminated = (boolean) (strchr (PathDelimiters, lastChar) != NULL);
-# else
-    boolean terminated = (boolean) (lastChar == PATH_SEPARATOR);
-# endif
-
-    vStringCopyS (filePath, path);
-    if (! terminated)
-    {
-	vStringPut (filePath, OUTPUT_PATH_SEPARATOR);
-	vStringTerminate (filePath);
-    }
-    vStringCatS (filePath, file);
-#endif
-
-    return filePath;
-}
-
-extern void addTotals (const unsigned int files,
-		       const long unsigned int lines,
-		       const long unsigned int bytes)
+extern void addTotals (
+	const unsigned int files, const long unsigned int lines,
+	const long unsigned int bytes)
 {
     Totals.files += files;
     Totals.lines += lines;
@@ -191,142 +138,110 @@ extern boolean isDestinationStdout (void)
     return toStdout;
 }
 
-/*
- *	Create tags
- */
-
-extern void processExcludeOption (const char *const __unused__ option,
-				  const char *const parameter)
-{
-    const char *const fileName = parameter + 1;
-    if (parameter [0] == '\0')
-	freeList (&Excluded);
-    else if (parameter [0] == '@')
-    {
-	stringList* const sl = stringListNewFromFile (fileName);
-	if (sl == NULL)
-	    error (FATAL | PERROR, "cannot open \"%s\"", fileName);
-	if (Excluded == NULL)
-	    Excluded = sl;
-	else
-	    stringListCombine (Excluded, sl);
-	verbose ("    adding exclude patterns from %s\n", fileName);
-    }
-    else
-    {
-	vString *const item = vStringNewInit (parameter);
-	if (Excluded == NULL)
-	    Excluded = stringListNew ();
-	stringListAdd (Excluded, item);
-	verbose ("    adding exclude pattern: %s\n", parameter);
-    }
-}
-
-static boolean excludedFile (const char* const name)
-{
-    const char* base = baseFilename (name);
-    boolean result = FALSE;
-    if (Excluded != NULL)
-    {
-	result = stringListFileMatched (Excluded, base);
-	if (! result  &&  name != base)
-	    result = stringListFileMatched (Excluded, name);
-    }
-#ifdef AMIGA
-    /* not a good solution, but the only one which works often */
-    if (! result)
-	result = (boolean) (strcmp (name, TagFile.name) == 0);
-#endif
-    return result;
-}
-
-# if defined (MSDOS) || defined (WIN32)
-
-static boolean createTagsForMatchingEntries (char *const pattern)
+#if defined (HAVE_OPENDIR)
+static boolean recurseUsingOpendir (const char *const dirName)
 {
     boolean resize = FALSE;
-    const size_t dirLength = baseFilename (pattern) - (char *) pattern;
-    vString *const filePath = vStringNew ();
+    DIR *const dir = opendir (dirName);
+    if (dir == NULL)
+	error (WARNING | PERROR, "cannot recurse into directory \"%s\"", dirName);
+    else
+    {
+	struct dirent *entry;
+	while ((entry = readdir (dir)) != NULL)
+	{
+	    if (strcmp (entry->d_name, ".") != 0  &&
+		strcmp (entry->d_name, "..") != 0)
+	    {
+		vString *filePath;
+		if (strcmp (dirName, ".") == 0)
+		    filePath = vStringNewInit (entry->d_name);
+		else
+		    filePath = combinePathAndFile (dirName, entry->d_name);
+		resize |= createTagsForEntry (vStringValue (filePath));
+		vStringDelete (filePath);
+	    }
+	}
+	closedir (dir);
+    }
+    return resize;
+}
+
+#elif defined (HAVE_FINDFIRST) || defined (HAVE__FINDFIRST)
+
+static boolean createTagsForWildcardEntry (
+	const char *const pattern, const size_t dirLength,
+	const char *const entryName)
+{
+    boolean resize = FALSE;
+    /* we must not recurse into the directories "." or ".." */
+    if (strcmp (entryName, ".") != 0  &&  strcmp (entryName, "..") != 0)
+    {
+	vString *const filePath = vStringNew ();
+	vStringNCopyS (filePath, pattern, dirLength);
+	vStringCatS (filePath, entryName);
+	resize = createTagsForEntry (vStringValue (filePath));
+	vStringDelete (filePath);
+    }
+    return resize;
+}
+
+static boolean createTagsForWildcardUsingFindfirst (const char *const pattern)
+{
+    boolean resize = FALSE;
+    const size_t dirLength = baseFilename (pattern) - pattern;
 #if defined (HAVE_FINDFIRST)
     struct ffblk fileInfo;
     int result = findfirst (pattern, &fileInfo, FA_DIREC);
-
     while (result == 0)
     {
-	const char *const entryName = fileInfo.ff_name;
-
-	/*  We must not recurse into the directories "." or "..".
-	 */
-	if (strcmp (entryName, ".") != 0  &&  strcmp (entryName, "..") != 0)
-	{
-	    vStringNCopyS (filePath, pattern, dirLength);
-	    vStringCatS (filePath, entryName);
-	    resize |= createTagsForEntry (vStringValue (filePath));
-	}
+	const char *const entry = (const char *) fileInfo.ff_name;
+	resize |= createTagsForWildcardEntry (pattern, dirLength, entry);
 	result = findnext (&fileInfo);
     }
 #elif defined (HAVE__FINDFIRST)
     struct _finddata_t fileInfo;
-    long hFile = _findfirst (pattern, &fileInfo);
-
+    findfirst_t hFile = _findfirst (pattern, &fileInfo);
     if (hFile != -1L)
     {
 	do
 	{
-	    const char *const entryName = fileInfo.name;
-
-	    /*  We must not recurse into the directories "." or "..".
-	     */
-	    if (strcmp (entryName, ".") != 0  &&  strcmp (entryName, "..") != 0)
-	    {
-		vStringNCopyS (filePath, pattern, dirLength);
-		vStringCatS (filePath, entryName);
-		resize |= createTagsForEntry (vStringValue (filePath));
-	    }
+	    const char *const entry = (const char *) fileInfo.name;
+	    resize |= createTagsForWildcardEntry (pattern, dirLength, entry);
 	} while (_findnext (hFile, &fileInfo) == 0);
 	_findclose (hFile);
     }
 #endif
-
-    vStringDelete (filePath);
     return resize;
 }
 
 #elif defined (AMIGA)
 
-static boolean createTagsForMatchingEntries (char *const pattern)
+static boolean createTagsForAmigaWildcard (const char *const pattern)
 {
     boolean resize = FALSE;
     struct AnchorPath *const anchor =
-			(struct AnchorPath *) eMalloc ((size_t) ANCHOR_SIZE);
+	    (struct AnchorPath *) eMalloc ((size_t) ANCHOR_SIZE);
+    LONG result;
 
-    if (anchor != NULL)
-    {
-	LONG result;
-
-	memset (anchor, 0, (size_t) ANCHOR_SIZE);
-	anchor->ap_Strlen = ANCHOR_BUF_SIZE; /* ap_Length no longer supported */
-
-	/*  Allow '.' for current directory.
-	 */
+    memset (anchor, 0, (size_t) ANCHOR_SIZE);
+    anchor->ap_Strlen = ANCHOR_BUF_SIZE;
+    /* Allow '.' for current directory */
 #ifdef APF_DODOT
-	anchor->ap_Flags = APF_DODOT | APF_DOWILD;
+    anchor->ap_Flags = APF_DODOT | APF_DOWILD;
 #else
-	anchor->ap_Flags = APF_DoDot | APF_DoWild;
+    anchor->ap_Flags = APF_DoDot | APF_DoWild;
 #endif
-
-	result = MatchFirst ((UBYTE *) pattern, anchor);
-	while (result == 0)
-	{
-	    resize |= createTagsForEntry ((char *) anchor->ap_Buf);
-	    result = MatchNext (anchor);
-	}
-	MatchEnd (anchor);
-	eFree (anchor);
+    result = MatchFirst ((UBYTE *) pattern, anchor);
+    while (result == 0)
+    {
+	resize |= createTagsForEntry ((char *) anchor->ap_Buf);
+	result = MatchNext (anchor);
     }
+    MatchEnd (anchor);
+    eFree (anchor);
     return resize;
 }
-
 #endif
 
 static boolean recurseIntoDirectory (const char *const dirName)
@@ -338,50 +253,32 @@ static boolean recurseIntoDirectory (const char *const dirName)
 	verbose ("ignoring \"%s\" (directory)\n", dirName);
     else
     {
-#if defined (HAVE_OPENDIR)
-	DIR *const dir = opendir (dirName);
-	if (dir == NULL)
-	    error (WARNING | PERROR, "cannot recurse into directory \"%s\"",
-		   dirName);
-	else
-	{
-	    struct dirent *entry;
-	    verbose ("RECURSING into directory \"%s\"\n", dirName);
-	    while ((entry = readdir (dir)) != NULL)
-	    {
-		if (strcmp (entry->d_name, ".") != 0  &&
-		    strcmp (entry->d_name, "..") != 0)
-		{
-		    vString *filePath;
-		    if (strcmp (dirName, ".") == 0)
-			filePath = vStringNewInit (entry->d_name);
-		    else
-			filePath = combinePathAndFile (dirName, entry->d_name);
-		    resize |= createTagsForEntry (vStringValue (filePath));
-		    vStringDelete (filePath);
-		}
-	    }
-	    closedir (dir);
-	}
-#elif defined (AMIGA) || defined (MSDOS) || defined (WIN32)
-	vString *const pattern = vStringNew ();
 	verbose ("RECURSING into directory \"%s\"\n", dirName);
-# ifdef AMIGA
-	if (*dirName != '\0'  &&  strcmp (dirName, ".") != 0)
+#if defined (HAVE_OPENDIR)
+	resize = recurseUsingOpendir (dirName);
+#elif defined (HAVE_FINDFIRST) || defined (HAVE__FINDFIRST)
 	{
+	    vString *const pattern = vStringNew ();
 	    vStringCopyS (pattern, dirName);
-	    if (dirName [strlen (dirName) - 1] != '/')
-		vStringPut (pattern, '/');
+	    vStringPut (pattern, OUTPUT_PATH_SEPARATOR);
+	    vStringCatS (pattern, "*.*");
+	    resize = createTagsForWildcardUsingFindfirst (vStringValue (pattern));
+	    vStringDelete (pattern);
 	}
-	vStringCatS (pattern, "#?");
-# else
-	vStringCopyS (pattern, dirName);
-	vStringPut (pattern, OUTPUT_PATH_SEPARATOR);
-	vStringCatS (pattern, "*.*");
-# endif
-	resize = createTagsForMatchingEntries (vStringValue (pattern));
-	vStringDelete (pattern);
-#endif	/* HAVE_OPENDIR */
+#elif defined (AMIGA)
+	{
+	    vString *const pattern = vStringNew ();
+	    if (*dirName != '\0'  &&  strcmp (dirName, ".") != 0)
+	    {
+		vStringCopyS (pattern, dirName);
+		if (dirName [strlen (dirName) - 1] != '/')
+		    vStringPut (pattern, '/');
+	    }
+	    vStringCatS (pattern, "#?");
+	    resize = createTagsForAmigaWildcard (vStringValue (pattern));
+	    vStringDelete (pattern);
+	}
+#endif
     }
     return resize;
 }
@@ -392,7 +289,7 @@ static boolean createTagsForEntry (const char *const entryName)
     fileStatus *status = eStat (entryName);
 
     Assert (entryName != NULL);
-    if (excludedFile (entryName))
+    if (isExcludedFile (entryName))
 	verbose ("excluding \"%s\"\n", entryName);
     else if (status->isSymbolicLink  &&  ! Option.followLinks)
 	verbose ("ignoring \"%s\" (symbolic link)\n", entryName);
@@ -408,6 +305,32 @@ static boolean createTagsForEntry (const char *const entryName)
     return resize;
 }
 
+#ifdef MANUAL_GLOBBING
+
+static boolean createTagsForWildcardArg (const char *const arg)
+{
+    boolean resize = FALSE;
+    vString *const pattern = vStringNewInit (arg);
+    char *patternS = vStringValue (pattern);
+
+#if defined (HAVE_FINDFIRST) || defined (HAVE__FINDFIRST)
+    /*  We must transform the "." and ".." forms into something that can
+     *  be expanded by the findfirst/_findfirst functions.
+     */
+    if (Option.recurse  &&
+	(strcmp (patternS, ".") == 0  ||  strcmp (patternS, "..") == 0))
+    {
+	vStringPut (pattern, OUTPUT_PATH_SEPARATOR);
+	vStringCatS (pattern, "*.*");
+    }
+    resize |= createTagsForWildcardUsingFindfirst (patternS);
+#endif
+    vStringDelete (pattern);
+    return resize;
+}
+
+#endif
+
 static boolean createTagsForArgs (cookedArgs* const args)
 {
     boolean resize = FALSE;
@@ -416,23 +339,10 @@ static boolean createTagsForArgs (cookedArgs* const args)
      */
     while (! cArgOff (args))
     {
-	const char *arg = cArgItem (args);
+	const char *const arg = cArgItem (args);
 
-#if defined (MSDOS) || defined (WIN32)
-	vString *const pattern = vStringNewInit (arg);
-	char *patternS = vStringValue (pattern);
-
-	/*  We must transform the "." and ".." forms into something that can
-	 *  be expanded by the MSDOS/Windows functions.
-	 */
-	if (Option.recurse  &&
-	    (strcmp (patternS, ".") == 0  ||  strcmp (patternS, "..") == 0))
-	{
-	    vStringPut (pattern, OUTPUT_PATH_SEPARATOR);
-	    vStringCatS (pattern, "*.*");
-	}
-	resize |= createTagsForMatchingEntries (patternS);
-	vStringDelete (pattern);
+#ifdef MANUAL_GLOBBING
+	resize |= createTagsForWildcardArg (arg);
 #else
 	resize |= createTagsForEntry (arg);
 #endif
@@ -557,7 +467,7 @@ static void makeTags (cookedArgs* args)
     boolean files = (boolean)(! cArgOff (args) || Option.fileList != NULL
 			      || Option.filter);
 
-    if (! files  &&  ! Option.recurse  &&  ! Option.etagsInclude)
+    if (filesRequired ()  && ! files)
 	error (FATAL, "No files specified. Try \"%s --help\".",
 	       getExecutableName ());
 
@@ -643,7 +553,6 @@ extern int main (int __unused__ argc, char **argv)
     /*  Clean up.
      */
     eFree (CurrentDirectory);
-    freeList (&Excluded);
     cArgDelete (args);
     freeKeywordTable ();
     freeSourceFileResources ();
