@@ -1,5 +1,5 @@
 /*
-*   $Id: php.c,v 1.1 2001/11/02 04:53:43 darren Exp $
+*   $Id: php.c,v 1.3 2003/07/18 02:32:29 darren Exp $
 *
 *   Copyright (c) 2000, Jesus Castagnetto <jmcastagnetto@zkey.com>
 *
@@ -9,6 +9,8 @@
 *   This module contains functions for generating tags for the PHP web page
 *   scripting language. Only recognizes functions and classes, not methods or
 *   variables.
+*
+*   Parsing PHP defines by Pavel Hlousek <pavel.hlousek@seznam.cz>, Apr 2003.
 */
 
 /*
@@ -26,11 +28,12 @@
 *   DATA DEFINITIONS
 */
 typedef enum {
-    K_CLASS, K_FUNCTION
+    K_CLASS, K_DEFINE, K_FUNCTION
 } phpKind;
 
 static kindOption PhpKinds [] = {
     { TRUE, 'c', "class",    "classes" },
+    { TRUE, 'd', "define",   "constant definitions" },
     { TRUE, 'f', "function", "functions" }
 };
 
@@ -67,10 +70,7 @@ static void findPhpTags (void)
 		++cp;
 	    }
 	    vStringTerminate (name);
-	    while (isspace ((int) *cp))
-		++cp;
-	    if (*cp++ == '(')
-		makeSimpleTag (name, PhpKinds, K_FUNCTION);
+	    makeSimpleTag (name, PhpKinds, K_FUNCTION);
 	    vStringClear (name);
 	} 
 	else if (strncmp ((const char*) cp, "class", (size_t) 5) == 0 &&
@@ -89,6 +89,33 @@ static void findPhpTags (void)
 	    makeSimpleTag (name, PhpKinds, K_CLASS);
 	    vStringClear (name);
 	}
+	else if (strncmp ((const char*) cp, "define", (size_t) 6) == 0 &&
+	         ! isalnum ((int) cp [6]))
+	{
+	    cp += 6;
+
+	    while (isspace ((int) *cp))
+		++cp;
+	    if (*cp != '(')
+		continue;
+	    ++cp;
+
+	    while (isspace ((int) *cp))
+		++cp;
+	    if ((*cp == '\'') || (*cp == '"'))
+		++cp;
+	    else if (! ((*cp == '_')  || isalnum ((int) *cp)))
+		continue;
+          
+	    while (isalnum ((int) *cp)  ||  *cp == '_')
+	    {
+		vStringPut (name, (int) *cp);
+		++cp;
+	    }
+	    vStringTerminate (name);
+	    makeSimpleTag (name, PhpKinds, K_DEFINE);
+	    vStringClear (name);
+	}
 
     }
     vStringDelete (name);
