@@ -1,7 +1,7 @@
 /*
-*   $Id: read.h,v 9.1 2001/04/11 05:14:18 darren Exp $
+*   $Id: read.h,v 1.4 2002/02/17 06:04:32 darren Exp $
 *
-*   Copyright (c) 1998-2001, Darren Hiebert
+*   Copyright (c) 1998-2002, Darren Hiebert
 *
 *   This source code is released for free distribution under the terms of the
 *   GNU General Public License.
@@ -42,14 +42,6 @@
 #define isLanguage(lang)	(boolean)((lang) == File.source.language)
 #define isHeaderFile()		File.source.isHeader
 
-/*  Is the character valid as a character of a C identifier?
- */
-#define isident(c)	(isalnum(c) || (c) == '_')
-
-/*  Is the character valid as the first character of a C identifier?
- */
-#define isident1(c)	(isalpha(c) || (c) == '_' || (c) == '~')
-
 /*
 *   DATA DECLARATIONS
 */
@@ -110,11 +102,6 @@ extern CONST_FILE inputFile File;
 /*
 *   FUNCTION PROTOTYPES
 */
-#ifdef NEED_PROTO_FGETPOS
-extern int fgetpos  (FILE *stream, fpos_t *pos);
-extern int fsetpos  (FILE *stream, const fpos_t *pos);
-#endif
-
 extern void freeSourceFileResources (void);
 extern boolean fileOpen (const char *const fileName, const langType language);
 extern boolean fileEOF (void);
