@@ -1,7 +1,7 @@
 /*
-*   $Id: readtags.c,v 1.16 2002/09/16 06:22:39 darren Exp $
+*   $Id: readtags.c,v 1.24 2003/07/20 18:46:30 darren Exp $
 *
-*   Copyright (c) 1996-2002, Darren Hiebert
+*   Copyright (c) 1996-2003, Darren Hiebert
 *
 *   This source code is released into the public domain.
 *
@@ -11,16 +11,13 @@
 /*
 *   INCLUDE FILES
 */
-#include "general.h"	/* must always come first */
-
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <errno.h>
-#ifdef HAVE_SYS_TYPES_H
-# include <sys/types.h>	/* declare off_t */
-#endif
 #include <stdio.h>
+#include <errno.h>
+#include <sys/types.h>	/* to declare off_t */
+
 #include "readtags.h"
 
 /*
@@ -44,7 +41,7 @@ struct sTagFile {
 	/* format of tag file */
     short format;
 	/* how is the tag file sorted? */
-    short sortMethod;
+    sortType sortMethod;
 	/* pointer to file structure */
     FILE* fp;
 	/* file position of first character of `line' */
@@ -236,7 +233,7 @@ static int readTagLine (tagFile *const file)
 static tagResult growFields (tagFile *const file)
 {
     tagResult result = TagFailure;
-    size_t newCount = 2 * file->fields.max;
+    unsigned short newCount = 2 * file->fields.max;
     tagExtensionField *newFields = (tagExtensionField*)
 	    realloc (file->fields.list, newCount * sizeof (tagExtensionField));
     if (newFields == NULL)
@@ -401,7 +398,7 @@ static void readPseudoTags (tagFile *const file, tagFileInfo *const info)
 	    key = entry.name + prefixLength;
 	    value = entry.file;
 	    if (strcmp (key, "TAG_FILE_SORTED") == 0)
-		file->sortMethod = atoi (value);
+		file->sortMethod = (sortType) atoi (value);
 	    else if (strcmp (key, "TAG_FILE_FORMAT") == 0)
 		file->format = atoi (value);
 	    else if (strcmp (key, "TAG_PROGRAM_AUTHOR") == 0)
@@ -520,7 +517,7 @@ static const char *readFieldValue (
 	result = entry->kind;
     else if (strcmp (key, "file") == 0)
 	result = EmptyString;
-    else for (i = 0  ;  i < entry->fields.count  &&  result != NULL  ;  ++i)
+    else for (i = 0  ;  i < entry->fields.count  &&  result == NULL  ;  ++i)
 	if (strcmp (entry->fields.list [i].key, key) == 0)
 	    result = entry->fields.list [i].value;
     return result;
@@ -711,12 +708,12 @@ static tagResult findNext (tagFile *const file, tagEntry *const entry)
 *  EXTERNAL INTERFACE
 */
 
-extern tagFile *tagsOpen (const char *filePath, tagFileInfo *info)
+extern tagFile *tagsOpen (const char *const filePath, tagFileInfo *const info)
 {
     return initialize (filePath, info);
 }
 
-extern tagResult tagsSetSortType (tagFile *file, sortType type)
+extern tagResult tagsSetSortType (tagFile *const file, const sortType type)
 {
     tagResult result = TagFailure;
     if (file != NULL  &&  file->initialized)
@@ -771,7 +768,7 @@ extern tagResult tagsFindNext (tagFile *const file, tagEntry *const entry)
     return result;
 }
 
-extern tagResult tagsClose (tagFile *file)
+extern tagResult tagsClose (tagFile *const file)
 {
     tagResult result = TagFailure;
     if (file != NULL  &&  file->initialized)
@@ -792,16 +789,16 @@ static const char *TagFileName = "tags";
 static const char *ProgramName;
 static int extensionFields;
 static int SortOverride;
-static int SortMethod;
+static sortType SortMethod;
 
 static void printTag (const tagEntry *entry)
 {
     int i;
-    boolean first = TRUE;
+    int first = 1;
     const char* separator = ";\"";
     const char* const empty = "";
 /* "sep" returns a value only the first time it is evaluated */
-#define sep (first ? (first = FALSE, separator) : empty)
+#define sep (first ? (first = 0, separator) : empty)
     printf ("%s\t%s\t%s",
 	entry->name, entry->file, entry->address.pattern);
     if (extensionFields)
@@ -887,7 +884,7 @@ extern int main (int argc, char **argv)
     ProgramName = argv [0];
     if (argc == 1)
     {
-	fprintf (errout, Usage, ProgramName);
+	fprintf (stderr, Usage, ProgramName);
 	exit (1);
     }
     for (i = 1  ;  i < argc  ;  ++i)
@@ -900,7 +897,7 @@ extern int main (int argc, char **argv)
 	}
 	else
 	{
-	    int j;
+	    size_t j;
 	    for (j = 1  ;  arg [j] != '\0'  ;  ++j)
 	    {
 		switch (arg [j])
@@ -920,7 +917,7 @@ extern int main (int argc, char **argv)
 			    TagFileName = argv [++i];
 			else
 			{
-			    fprintf (errout, Usage, ProgramName);
+			    fprintf (stderr, Usage, ProgramName);
 			    exit (1);
 			}
 			break;
@@ -928,30 +925,30 @@ extern int main (int argc, char **argv)
 			SortOverride = 1;
 			++j;
 			if (arg [j] == '\0')
-			    SortMethod = 1;
+			    SortMethod = TAG_SORTED;
 			else if (strchr ("012", arg[j]) != NULL)
-			    SortMethod = arg[j] - '0';
+			    SortMethod = (sortType) (arg[j] - '0');
 			else
 			{
-			    fprintf (errout, Usage, ProgramName);
+			    fprintf (stderr, Usage, ProgramName);
 			    exit (1);
 			}
 			break;
 		    default:
-			fprintf (errout, "%s: unknown option: %c\n",
+			fprintf (stderr, "%s: unknown option: %c\n",
 				    ProgramName, arg[j]);
 			exit (1);
 			break;
 		}
 	    }
 	}
-	if (! actionSupplied)
-	{
-	    fprintf (errout,
-		"%s: no action specified: specify tag name(s) or -l option\n",
-		ProgramName);
-	    exit (1);
-	}
+    }
+    if (! actionSupplied)
+    {
+	fprintf (stderr,
+	    "%s: no action specified: specify tag name(s) or -l option\n",
+	    ProgramName);
+	exit (1);
     }
     return 0;
 }
