@@ -1,7 +1,7 @@
 /*
-*   $Id: readtags.h,v 1.8 2002/09/16 06:22:39 darren Exp $
+*   $Id: readtags.h,v 1.12 2003/04/01 05:02:21 darren Exp $
 *
-*   Copyright (c) 1996-2002, Darren Hiebert
+*   Copyright (c) 1996-2003, Darren Hiebert
 *
 *   This source code is released for the public domain.
 *
@@ -21,6 +21,10 @@
 #ifndef READTAGS_H
 #define READTAGS_H
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /*
 *  MACROS
 */
@@ -151,7 +155,7 @@ typedef struct {
 *  be set to the errno value representing the system error preventing the tag
 *  file from being successfully opened.
 */
-extern tagFile *tagsOpen (const char *filePath, tagFileInfo *info);
+extern tagFile *tagsOpen (const char *const filePath, tagFileInfo *const info);
 
 /*
 *  This function allows the client to override the normal automatic detection
@@ -166,7 +170,7 @@ extern tagFile *tagsOpen (const char *filePath, tagFileInfo *info);
 *  it actually is not. The function will return TagSuccess if called on an
 *  open tag file or TagFailure if not.
 */
-extern tagResult tagsSetSortType (tagFile *file, sortType type);
+extern tagResult tagsSetSortType (tagFile *const file, const sortType type);
 
 /*
 *  Reads the first tag in the file, if any. It is passed the handle to an
@@ -175,7 +179,7 @@ extern tagResult tagsSetSortType (tagFile *file, sortType type);
 *  The function will return TagSuccess another tag entry is found, or
 *  TagFailure if not (i.e. it reached end of file).
 */
-extern tagResult tagsFirst (tagFile *file, tagEntry *entry);
+extern tagResult tagsFirst (tagFile *const file, tagEntry *const entry);
 
 /*
 *  Step to the next tag in the file, if any. It is passed the handle to an
@@ -185,7 +189,7 @@ extern tagResult tagsFirst (tagFile *file, tagEntry *entry);
 *  if not (i.e. it reached end of file). It will always read the first tag in
 *  the file immediately after calling tagsOpen().
 */
-extern tagResult tagsNext (tagFile *file, tagEntry *entry);
+extern tagResult tagsNext (tagFile *const file, tagEntry *const entry);
 
 /*
 *  Retrieve the value associated with the extension field for a specified key.
@@ -194,7 +198,7 @@ extern tagResult tagsNext (tagFile *file, tagEntry *entry);
 *  containing the key of the desired extension field. If no such field of the
 *  specified key exists, the function will return null.
 */
-extern const char *tagsField (const tagEntry *entry, const char *key);
+extern const char *tagsField (const tagEntry *const entry, const char *const key);
 
 /*
 *  Find the first tag matching `name'. The structure pointed to by `entry'
@@ -221,7 +225,7 @@ extern const char *tagsField (const tagEntry *entry, const char *key);
 *  The function will return TagSuccess if a tag matching the name is found, or
 *  TagFailure if not.
 */
-extern tagResult tagsFind (tagFile *file, tagEntry *entry, const char *name, int options);
+extern tagResult tagsFind (tagFile *const file, tagEntry *const entry, const char *const name, const int options);
 
 /*
 *  Find the next tag matching the name and options supplied to the most recent
@@ -230,14 +234,18 @@ extern tagResult tagsFind (tagFile *file, tagEntry *entry, const char *name, int
 *  function will return TagSuccess if another tag matching the name is found,
 *  or TagFailure if not.
 */
-extern tagResult tagsFindNext (tagFile *file, tagEntry *entry);
+extern tagResult tagsFindNext (tagFile *const file, tagEntry *const entry);
 
 /*
 *  Call tagsTerminate() at completion of reading the tag file, which will
 *  close the file and free any internal memory allocated. The function will
 *  return TagFailure is no file is currently open, TagSuccess otherwise.
 */
-extern tagResult tagsClose (tagFile *file);
+extern tagResult tagsClose (tagFile *const file);
+
+#ifdef __cplusplus
+};
+#endif
 
 #endif
 
